from aio_manager import Command


class HelloWorld(Command):
    """
    Prints 'Hello world' message
    """
    def __init__(self, app):
        super().__init__('say_hello', app)

    def run(self, app, args):
        print('Hello world')



import code
import logging
import asyncio
import os

from aio_manager import Command


class Shell(Command):
    """
    Command to run the shell.
    """

    banner = 'AIOManager Shell. The "app" variable is available in the context.'

    def __init__(self, app):
        super().__init__('shell', app)

    def run(self, app, args):
        """
        Logic copied from https://github.com/smurfix/flask-script/blob/master/flask_script/commands.py
        """
        context = self.create_context(app)

        if not args.no_ptipython:
            # Try PtIPython
            try:
                from ptpython.ipython import embed
                history_filename = os.path.expanduser('~/.ptpython_history')
                embed(banner1=self.banner, user_ns=context, history_filename=history_filename)
                return
            except ImportError:
                pass

        if not args.no_ptpython:
            # Try PtPython
            try:
                from ptpython.repl import embed
                history_filename = os.path.expanduser('~/.ptpython_history')
                embed(globals=context, history_filename=history_filename)
                return
            except ImportError:
                pass

        if not args.no_bpython:
            # Try BPython
            try:
                from bpython import embed
                embed(banner=self.banner, locals_=context)
                return
            except ImportError:
                pass

        if not args.no_ipython:
            # Try IPython
            try:
                from IPython import embed
                embed(banner1=self.banner, user_ns=context)
                return
            except ImportError:
                pass

        code.interact(self.banner, local=context)

    def create_context(self, app):
        return {'app': app}

    def configure_parser(self, parser):
        super().configure_parser(parser)
        parser.add_argument('--no-ipython',
               action="store_true",
               dest='no_ipython',
               help="Do not use the IPython shell")
        parser.add_argument('--no-bpython',
               action="store_true",
               dest='no_bpython',
               help="Do not use the BPython shell")
        parser.add_argument('--no-ptipython',
               action="store_true",
               dest='no_ptipython',
               help="Do not use the PtIPython shell")
        parser.add_argument('--no-ptpython',
               action="store_true",
               dest='no_ptpython',
               help="Do not use the PtPython shell")



"""
Inspired by Flask-Script and Django's manage.py
"""

import argparse
import abc


class BaseManager:
    def __init__(self):
        self.commands = []

    def add_command(self, command):
        self.commands.append(command)

    def create_parser(self):
        base_parser = argparse.ArgumentParser(add_help=True)
        subparsers = base_parser.add_subparsers()
        for command in self.commands:
            subparser = subparsers.add_parser(command.name,
                                              help=command.__doc__,
                                              add_help=True)
            command.configure_parser(subparser)
            subparser.set_defaults(command=command)
        return base_parser

    def run(self):
        parser = self.create_parser()
        parsed = parser.parse_args()
        if hasattr(parsed, 'command'):
            parsed.command.execute(parsed)
        else:
            parser.print_help()


class Manager(BaseManager):
    def __init__(self, app=None):
        super().__init__()
        self.app = app

        from .commands.runserver import RunServer
        from .commands.shell import Shell
        self.add_command(RunServer(self.app))
        self.add_command(Shell(self.app))


class Command(metaclass=abc.ABCMeta):
    def __init__(self, name, app):
        self.name = name
        self.app = app

    def connect_app(self, app):
        self.app = app

    def execute(self, args):
        if self.app is None:
            raise RuntimeError("Command '{}' is not connected to Application".format(self.name))
        self.run(self.app, args)

    @abc.abstractmethod
    def run(self, app, args):
        pass

    def configure_parser(self, parser):
        pass



import asyncio
import logging
from aio_manager import Command
from aiopg.sa import create_engine
from colorama import Back, Style, Fore
from sqlalchemy import event

logger = logging.getLogger(__name__)


class SACommand(Command):
    def __init__(self, name, app, declarative_base, user, database, host, password):
        super().__init__(name, app)
        self.declarative_base = declarative_base
        self.user = user
        self.database = database
        self.host = host
        self.password = password

    @asyncio.coroutine
    def create_engine(self):
        engine = yield from create_engine(user=self.user,
                                          database=self.database,
                                          host=self.host,
                                          password=self.password,
                                          echo=True)
        return engine


class CreateTables(SACommand):
    """
    Creates DB tables for all models
    """

    def __init__(self, *args, **kwargs):
        super().__init__('init', *args, **kwargs)

    @asyncio.coroutine
    def create_tables(self):
        def receive_after_drop(target, connection, **kw):
            print('  ' + target.name + Fore.GREEN + ' created ' + Style.RESET_ALL)

        for name, table in self.declarative_base.metadata.tables.items():
            event.listen(table, 'after_create', receive_after_drop)

        print(Fore.GREEN + 'Creating all tables' + Style.RESET_ALL)
        engine = yield from self.create_engine()
        yield from self.declarative_base.metadata.create_all(engine)

    def run(self, app, args):
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.create_tables())


class DropTables(SACommand):
    """
    Drops DB tables for all models
    """
    def __init__(self, *args, **kwargs):
        super().__init__('drop_tables', *args, **kwargs)

    @asyncio.coroutine
    def drop_tables(self):
        def receive_after_drop(target, connection, **kw):
            print('  ' + target.name + Fore.RED + ' dropped ' + Style.RESET_ALL)

        for name, table in self.declarative_base.metadata.tables.items():
            event.listen(table, 'after_drop', receive_after_drop)

        print(Fore.RED + 'Dropping all tables' + Style.RESET_ALL)
        engine = yield from self.create_engine()
        yield from self.declarative_base.metadata.drop_all(engine)

    def run(self, app, args):
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.drop_tables())


def configure_manager(manager, app, declarative_base, user, database, host, password):
    manager.add_command(CreateTables(app, declarative_base,
                                     user, database,
                                     host, password))
    manager.add_command(DropTables(app, declarative_base,
                                   user, database,
                                   host, password))



from setuptools import setup, find_packages


classifiers = [
    'License :: OSI Approved :: BSD License',
    'Intended Audience :: Developers',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Environment :: Web Environment',
    'Development Status :: 4 - Beta',
]


setup(name='aio_manager',
      version='0.1.2',
      description='Script manager for aiohttp.',
      long_description=('Script manager for aiohttp.\n'
                        'Inspired by Flask-script. Allows to write external scripts. '),
      classifiers=classifiers,
      platforms=['POSIX'],
      author='Roman Rader',
      author_email='antigluk@gmail.com',
      url='https://github.com/rrader/aio_manager',
      license='BSD',
      packages=find_packages(),
      install_requires=['aiohttp', 'colorama'],
      extras_require={'sa': ['sqlalchemy>=0.9'], },
      provides=['aio_manager'],
      include_package_data = True,
      test_suite="tests.test_manager")



import asyncio
from aio_manager import Command
import logging


class RunServer(Command):
    """
    Creates aiohttp server coroutine and executes event loop
    """
    def __init__(self, app):
        super().__init__('run_server', app)

    @asyncio.coroutine
    def run_server(self, app, loop, host, port):
        handler = app.make_handler()
        srv = yield from loop.create_server(handler, host, port)
        logging.getLogger().info('Server started at http://{}:{}'.format(host, port))
        return srv, handler

    def run(self, app, args):
        logging.basicConfig(level=args.level)
        logging.getLogger().setLevel(args.level)

        loop = asyncio.get_event_loop()
        srv, handler = loop.run_until_complete(self.run_server(app, loop, args.hostname, args.port))
        try:
            loop.run_forever()
        except KeyboardInterrupt:
            loop.run_until_complete(handler.finish_connections())

    def configure_parser(self, parser):
        super().configure_parser(parser)
        parser.add_argument('--hostname', metavar='HOST',
                            help='host or ip to listen on')
        parser.add_argument('--port', type=int, metavar='PORT',
                            help='port to listening')
        parser.add_argument('--level', type=str, metavar='LEVEL',
                            help='logging level')
        parser.set_defaults(hostname='127.0.0.1', port=5000, level='INFO')
